from .base import Base


class Brand(Base):
    endpoint = '/brand'

    def get_brand_image(self):
        return self.client.get(
            self.endpoint + '/image'
        )

    def upload_brand_image(self, files):
        return self.client.post(
            self.endpoint + '/image',
            files=files
        )



import asyncio
import json
import logging
import ssl

import websockets

log = logging.getLogger('mattermostdriver.websocket')
log.setLevel(logging.INFO)


class Websocket:
    def __init__(self, options, token):
        self.options = options
        self._token = token

    @asyncio.coroutine
    def connect(self, event_handler):
        """
        Connect to the websocket and authenticate it.
        When the authentication has finished, start the loop listening for messages,
        sending a ping to the server to keep the connection alive.
        :param event_handler: Every websocket event will be passed there
        :type event_handler: Function
        :return:
        """
        context = ssl.create_default_context(purpose=ssl.Purpose.CLIENT_AUTH)
        if not self.options['verify']:
            context.verify_mode = ssl.CERT_NONE

        scheme = 'wss://'
        if self.options['scheme'] != 'https':
            scheme = 'ws://'
            context = None

        url = scheme + self.options['url'] + ':' + str(self.options['port']) + self.options['basepath'] + '/websocket'

        websocket = yield from websockets.connect(
            url,
            ssl=context,
        )

        yield from self._authenticate_websocket(websocket, event_handler)
        yield from self._start_loop(websocket, event_handler)

    @asyncio.coroutine
    def _start_loop(self, websocket, event_handler):
        """
        We will listen for websockets events, sending a heartbeat/pong everytime
        we react a TimeoutError. If we don't the webserver would close the idle connection,
        forcing us to reconnect.
        """
        log.debug('Starting websocket loop')
        while True:
            try:
                yield from asyncio.wait_for(
                    self._wait_for_message(websocket, event_handler),
                    timeout=self.options['timeout']
                )
            except asyncio.TimeoutError:
                yield from websocket.pong()
                log.debug("Sending heartbeat...")
                continue

    @asyncio.coroutine
    def _authenticate_websocket(self, websocket, event_handler):
        """
        Sends a authentication challenge over a websocket.
        This is not needed when we just send the cookie we got on login
        when connecting to the websocket.
        """
        json_data = json.dumps({
            "seq": 1,
            "action": "authentication_challenge",
            "data": {
                "token": self._token
            }
        }).encode('utf8')
        yield from websocket.send(json_data)
        while True:
            message = yield from websocket.recv()
            status = json.loads(message)
            log.debug(status)
            # We want to pass the events to the event_handler already
            # because the hello event could arrive before the authentication ok response
            yield from event_handler(message)
            if ('status' in status and status['status'] == 'OK') and \
                    ('seq_reply' in status and status['seq_reply'] == 1):
                log.info('Websocket authentification OK')
                return True

    @asyncio.coroutine
    def _wait_for_message(self, websocket, event_handler):
        while True:
            message = yield from websocket.recv()
            yield from event_handler(message)



from .base import Base


class Files(Base):
    endpoint = '/files'

    def upload_file(self, channel_id, files):
        return self.client.post(
            self.endpoint,
            data={'channel_id': channel_id},
            files=files
        )

    def get_file(self, file_id):
        return self.client.get(
            self.endpoint + '/' + file_id,
        )

    def get_file_thumbnail(self, file_id):
        return self.client.get(
            self.endpoint + '/' + file_id + '/thumbnail',
        )

    def get_file_preview(self, file_id):
        return self.client.get(
            self.endpoint + '/' + file_id + '/preview',
        )

    def get_public_file_link(self, file_id):
        return self.client.get(
            self.endpoint + '/' + file_id + '/link',
        )

    def get_file_metadata(self, file_id):
        return self.client.get(
            self.endpoint + '/' + file_id + '/info',
        )



from .base import Base
from .users import Users


class Preferences(Base):
    def get_user_preferences(self, user_id):
        return self.client.get(
            Users.endpoint + '/' + user_id + '/preferences'
        )

    def save_user_preferences(self, user_id, options=None):
        return self.client.put(
            Users.endpoint + '/' + user_id + '/preferences',
            options=options
        )

    def delete_user_preferences(self, user_id, options=None):
        return self.client.post(
            Users.endpoint + '/' + user_id + '/preferences/delete',
            options=options
        )

    def list_user_preferences_by_category(self, user_id, category):
        return self.client.get(
            Users.endpoint + '/' + user_id + '/preferences/' + category
        )

    def get_specific_user_preference(self, user_id, category, preference_name):
        return self.client.get(
            Users.endpoint + '/' + user_id + '/preferences/' + category + '/name/' + preference_name
        )



from .base import Base


class Elasticsearch(Base):
    endpoint = '/elasticsearch'

    def test_elasticsearch_configuration(self):
        return self.client.post(
            self.endpoint + '/test'
        )

    def purge_all_elasticsearch_indexes(self):
        return self.client.post(
            self.endpoint + '/purge_indexes'
        )



import warnings

from .base import Base
from .users import Users


class Teams(Base):
    endpoint = '/teams'

    def create_team(self, options=None):
        return self.client.post(
            self.endpoint,
            options
        )

    def get_teams(self, params=None):
        return self.client.get(
            self.endpoint,
            params=params
        )

    def get_team(self, team_id):
        return self.client.get(
            self.endpoint + '/' + team_id,
        )

    def update_team(self, team_id, options=None):
        return self.client.put(
            self.endpoint + '/' + team_id,
            options
        )

    def delete_team(self, team_id, params=None):
        return self.client.delete(
            self.endpoint + '/' + team_id,
            params=params
        )

    def patch_team(self, team_id, options=None):
        return self.client.put(
            self.endpoint + '/' + team_id + '/patch',
            options
        )

    def get_team_by_name(self, name):
        return self.client.get(
            self.endpoint + '/name/' + name
        )

    def search_teams(self, options=None):
        return self.client.post(
            self.endpoint + '/search',
            options
        )

    def check_team_exists(self, name):
        return self.client.get(
            self.endpoint + '/name/' + name + '/exists'
        )

    def get_user_teams(self, user_id):
        return self.client.get(
            Users.endpoint + '/' + user_id + '/teams'
        )

    def get_team_members(self, team_id, params=None):
        return self.client.get(
            self.endpoint + '/' + team_id + '/members',
            params=params
        )

    def add_user_to_team(self, team_id, options=None):
        return self.client.post(
            self.endpoint + '/' + team_id + '/members',
            options=options
        )

    def add_user_to_team_from_invite(self, params=None):
        return self.client.post(
            self.endpoint + '/members/invite',
            params=params
        )

    def add_multiple_users_to_team(self, team_id, options=None):
        return self.client.post(
            self.endpoint + '/' + team_id + '/members/batch',
            options=options
        )

    def get_team_members_for_user(self, user_id):
        return self.client.get(
            Users.endpoint + '/' + user_id + '/teams/members'
        )

    def get_team_member(self, team_id, user_id):
        return self.client.get(
            self.endpoint + '/' + team_id + '/members/' + user_id
        )

    def remove_user_from_team(self, team_id, user_id):
        return self.client.delete(
            self.endpoint + '/' + team_id + '/members/' + user_id
        )

    def get_team_members_by_id(self, team_id, options=None):
        return self.client.post(
            self.endpoint + '/' + team_id + '/members/ids',
            options
        )

    def get_team_stats(self, team_id):
        return self.client.get(
            self.endpoint + '/' + team_id + '/stats'
        )

    def update_team_member_roles(self, team_id, user_id, options=None):
        return self.client.put(
            self.endpoint + '/' + team_id + '/members/' + user_id + '/roles',
            options
        )

    def get_team_unreads_for_user(self, user_id, params=None):
        return self.client.get(
            Users.endpoint + '/' + user_id + '/teams/unread',
            params=params
        )

    def get_unreads_for_team(self, user_id, team_id):
        return self.client.get(
            Users.endpoint + '/' + user_id + '/teams/' + team_id + '/unread',
        )

    def invite_users_to_team_by_mail(self, team_id, options=None):
        return self.client.post(
            self.endpoint + '/' + team_id + '/invite/email',
            options
        )

    def import_team_from_other_app(self, team_id, data=None):
        return self.client.post(
            self.endpoint + '/' + team_id + '/import',
            data=data
        )

    def get_invite_info_for_team(self, invite_id):
        return self.client.get(
            self.endpoint + '/invite/' + invite_id,
        )

    def get_public_channels(self, team_id, params=None):
        warnings.warn(
            'Using deprecated endpoint Teams.get_public_channels(). Use Channels.get_public_channels() instead.',
            DeprecationWarning
        )
        return self.client.get(
            self.endpoint + '/' + team_id + '/channels',
            params=params
        )

    def get_deleted_channels(self, team_id, params=None):
        warnings.warn(
            'Using deprecated endpoint Teams.get_deleted_channels(). Use Channels.get_deleted_channels() instead.',
            DeprecationWarning
        )
        return self.client.get(
            self.endpoint + '/' + team_id + '/channels/deleted',
            params=params
        )

    def search_channels(self, team_id, options=None):
        warnings.warn(
            'Using deprecated endpoint Teams.search_channels(). Use Channels.search_channels() instead.',
            DeprecationWarning
        )
        return self.client.post(
            self.endpoint + '/' + team_id + '/channels/search',
            options=options
        )



from .base import Base
from .users import Users


class OAuth(Base):
    endpoint = '/oauth'

    def register_oauth_app(self, options):
        return self.client.post(
            self.endpoint + '/apps',
            options=options
        )

    def get_oauth_apps(self, params=None):
        return self.client.get(
            self.endpoint + '/apps',
            params=params
        )

    def get_oauth_app(self, app_id):
        return self.client.get(
            self.endpoint + '/apps/' + app_id
        )

    def delete_oauth_app(self, app_id):
        return self.client.delete(
            self.endpoint + '/apps/' + app_id
        )

    def regenerate_oauth_app_secret(self, app_id):
        return self.client.post(
            self.endpoint + '/apps/' + app_id + '/regen_secret'
        )

    def get_info_on_oauth_app(self, app_id):
        return self.client.get(
            self.endpoint + '/apps/' + app_id + '/info'
        )

    def get_authorized_oauth_apps(self, user_id, params=None):
        return self.client.get(
            Users.endpoint + user_id + '/oauth/apps/authorized',
            params=params
        )



from .base import Base


class System(Base):
    def check_system_health(self):
        return self.client.get(
            '/system/ping'
        )

    def recycle_database_connection(self):
        return self.client.post(
            '/database/recycle'
        )

    def send_test_email(self):
        return self.client.post(
            '/email/test'
        )

    def get_configuration(self):
        return self.client.get(
            '/config'
        )

    def update_configuration(self):
        return self.client.put(
            '/config'
        )

    def reload_configuration(self):
        return self.client.post(
            '/config/reload'
        )

    def get_client_configuration(self, params):
        return self.client.get(
            '/config/client',
            params=params
        )

    def upload_license_file(self, files):
        return self.client.post(
            '/license',
            files=files
        )

    def remove_license_file(self):
        return self.client.delete(
            '/license'
        )

    def get_client_license(self, params):
        return self.client.get(
            '/license/client',
            params=params
        )

    def get_audits(self, params):
        return self.client.get(
            '/audits',
            params=params
        )

    def invalidate_all_caches(self):
        return self.client.post(
            '/caches/invalidate',
        )

    def get_logs(self, params):
        return self.client.get(
            '/logs',
            params=params
        )

    def add_log_message(self, options):
        return self.client.post(
            '/logs',
            options=options
        )

    def get_webrtc_token(self):
        return self.client.get(
            '/webrtc/token'
        )

    def get_analytics(self, params):
        return self.client.get(
            '/analytics/old',
            params=params
        )



from aiohttp import ClientError


class InvalidOrMissingParameters(ClientError):
    """
    Raised when mattermost returns a
    400 Invalid or missing parameters in URL or request body
    """


class NoAccessTokenProvided(ClientError):
    """
    Raised when mattermost returns a
    401 No access token provided
    """


class NotEnoughPermissions(ClientError):
    """
    Raised when mattermost returns a
    403 Do not have appropriate permissions
    """


class ResourceNotFound(ClientError):
    """
    Raised when mattermost returns a
    404 Resource not found
    """


class ContentTooLarge(ClientError):
    """
    Raised when mattermost returns a
    413 Content too large
    """


class FeatureDisabled(ClientError):
    """
    Raised when mattermost returns a
    501 Feature is disabled
    """



from .base import Base


class Emoji(Base):
    endpoint = '/emoji'

    def create_custom_emoji(self, emoji_name, files):
        return self.client.post(
            self.endpoint,
            data={'emoji': {'name': emoji_name}},
            files=files
        )

    def get_emoji_list(self, params=None):
        return self.client.get(
            self.endpoint,
            params=params
        )

    def get_custom_emoji(self, emoji_id):
        return self.client.get(
            self.endpoint + '/' + emoji_id
        )

    def delete_custom_emoji(self, emoji_id):
        return self.client.delete(
            self.endpoint + '/' + emoji_id
        )

    def get_custom_emoji_image(self, emoji_id):
        return self.client.get(
            self.endpoint + '/' + emoji_id + '/image'
        )



from .base import Base


class Webhooks(Base):
    endpoint = '/hooks'

    def create_incoming_hook(self, options):
        return self.client.post(
            self.endpoint + '/incoming',
            options=options
        )

    def list_incoming_hooks(self, params):
        return self.client.get(
            self.endpoint + '/incoming',
            params=params
        )

    def get_incoming_hook(self, hook_id):
        return self.client.get(
            self.endpoint + '/incoming/' + hook_id
        )

    def update_incoming_hook(self, hook_id, options):
        return self.client.put(
            self.endpoint + '/incoming/' + hook_id,
            options=options
        )

    def create_outgoing_hook(self, options):
        return self.client.post(
            self.endpoint + '/outgoing',
            options=options
        )

    def list_outgoing_hooks(self, params):
        return self.client.get(
            self.endpoint + '/outgoing',
            params=params
        )

    def get_outgoing_hook(self, hook_id):
        return self.client.get(
            self.endpoint + '/outgoing/' + hook_id
        )

    def delete_outgoing_hook(self, hook_id):
        return self.client.delete(
            self.endpoint + '/outgoing/' + hook_id
        )

    def update_outgoing_hook(self, hook_id, options):
        return self.client.put(
            self.endpoint + '/outgoing/' + hook_id,
            options=options
        )

    def regenerate_token_outgoing_hook(self, hook_id):
        return self.client.put(
            self.endpoint + '/outgoing/' + hook_id + '/regen_token'
        )

    def call_webhook(self, hook_id, options=None):
        return self.client.make_request('post', '/' + hook_id, options=options, basepath='/hooks')



from .base import Base
from .teams import Teams


class Commands(Base):
    endpoint = '/commands'

    def create_command(self, options=None):
        return self.client.post(
            self.endpoint,
            options=options
        )

    def list_commands_for_team(self, params=None):
        return self.client.get(
            self.endpoint,
            params=params
        )

    def list_autocomplete_commands(self, team_id):
        return self.client.get(
            Teams.endpoint + team_id + '/commands/autocomplete'
        )

    def update_command(self, command_id, options=None):
        return self.client.put(
            self.endpoint + command_id,
            options=options
        )

    def delete_command(self, command_id):
        return self.client.delete(
            self.endpoint + command_id
        )

    def generate_new_token(self, command_id):
        return self.client.put(
            self.endpoint + command_id + '/regen_token'
        )

    def execute_command(self, options=None):
        return self.client.post(
            self.endpoint + '/execute',
            options=options
        )



import logging

import asyncio
import aiohttp
import async_timeout

from .exceptions import (
    InvalidOrMissingParameters,
    NoAccessTokenProvided,
    NotEnoughPermissions,
    ContentTooLarge,
    FeatureDisabled
)

log = logging.getLogger('mattermostdriver.websocket')
log.setLevel(logging.INFO)


class Client:
    def __init__(self, options):
        self._url = '{scheme:s}://{url:s}:{port:d}{basepath:s}'.format(
            scheme=options['scheme'],
            url=options['url'],
            port=options['port'],
            basepath=options['basepath']
        )
        self._scheme = options['scheme']
        self._basepath = options['basepath']
        self._port = options['port']
        self._verify = options['verify']
        self._options = options
        self._token = ''
        self._cookies = None
        self._userid = ''
        self._username = ''
        self._session = aiohttp.ClientSession()

    @property
    def userid(self):
        """
        :return: The user id of the logged in user
        """
        return self._userid

    @userid.setter
    def userid(self, user_id):
        self._userid = user_id

    @property
    def username(self):
        """
        :return: The username of the logged in user. If none, returns an emtpy string.
        """
        return self._username

    @username.setter
    def username(self, username):
        self._username = username

    @property
    def url(self):
        return self._url

    @property
    def cookies(self):
        """
        :return: The cookie given on login
        """
        return self._cookies

    @cookies.setter
    def cookies(self, cookies):
        self._cookies = cookies

    @property
    def token(self):
        """
        :return: The token for the login
        """
        return self._token

    @token.setter
    def token(self, t):
        self._token = t

    def auth_header(self):
        if self._token == '':
            return {}
        return {"Authorization": "Bearer {token:s}".format(token=self._token)}

    @asyncio.coroutine
    def make_request(self, method, endpoint, options=None, params=None, data=None, files=None, basepath=None):
        kwargs = {}

        if options is not None:
            kwargs['json'] = options
        if params is not None:
            kwargs['params'] = params
        if data is not None and options is None:
            kwargs['data'] = data
        if files is not None and options is None:
            _data = kwargs.get('data', {})
            data = aiohttp.FormData()

            for k, v in files.items():
                data.add_field(k, v, content_type='application/octet-stream')

            for k, v in _data.items():
                data.add_field(k, v)

            kwargs['data'] = data

        if data is None:
            data = {}

        if basepath:
            url = '{scheme:s}://{url:s}:{port:d}{basepath:s}'.format(
                scheme=self._options['scheme'],
                url=self._options['url'],
                port=self._options['port'],
                basepath=basepath
            )
        else:
            url = self.url
        method = method.lower()
        request = self._session.get
        if method == 'post':
            request = self._session.post
        elif method == 'put':
            request = self._session.put
        elif method == 'delete':
            request = self._session.delete

        try:
            with async_timeout.timeout(self._options['timeout']):
                response = yield from request(
                    url + endpoint,
                    headers=self.auth_header(),
                    verify_ssl=self._verify,
                    **kwargs
                )
                response.raise_for_status()
        except aiohttp.ClientResponseError as e:
            if e.code == 400:
                raise InvalidOrMissingParameters(data['message'])
            elif e.code == 401:
                raise NoAccessTokenProvided(data['message'])
            elif e.code == 403:
                raise NotEnoughPermissions(data['message'])
            elif e.code == 413:
                raise ContentTooLarge(data['message'])
            elif e.code == 501:
                raise FeatureDisabled(data['message'])
            else:
                raise

        return response

    @asyncio.coroutine
    def get(self, endpoint, options=None, params=None):
        response = yield from self.make_request('get', endpoint, options=options, params=params)
        try:
            return (yield from response.json())
        except ValueError:
            return response

    @asyncio.coroutine
    def post(self, endpoint, options=None, params=None, data=None, files=None):
        response = yield from self.make_request('post', endpoint, options=options, params=params, data=data, files=files)
        return (yield from response.json())

    @asyncio.coroutine
    def put(self, endpoint, options=None, params=None, data=None):
        response = yield from self.make_request('put', endpoint, options=options, params=params, data=data)
        return (yield from response.json())

    @asyncio.coroutine
    def delete(self, endpoint, options=None, params=None, data=None):
        response = yield from self.make_request('delete', endpoint, options=options, params=params, data=data)
        return (yield from response.json())



from .base import Base
from .channels import Channels
from .teams import Teams
from .users import Users


class Posts(Base):
    endpoint = '/posts'

    def create_post(self, options):
        return self.client.post(
            self.endpoint,
            options=options
        )

    def get_post(self, post_id):
        return self.client.get(
            self.endpoint + '/' + post_id,
        )

    def delete_post(self, post_id):
        return self.client.delete(
            self.endpoint + '/' + post_id,
        )

    def update_post(self, post_id, options=None):
        return self.client.put(
            self.endpoint + '/' + post_id,
            options=options
        )

    def patch_post(self, post_id, options=None):
        return self.client.put(
            self.endpoint + '/' + post_id + '/patch',
            options=options
        )

    def get_thread(self, post_id):
        return self.client.get(
            self.endpoint + '/' + post_id + '/thread',
        )

    def get_list_of_flagged_posts(self, user_id, params=None):
        return self.client.get(
            Users.endpoint + '/' + user_id + '/posts/flagged',
            params=params
        )

    def get_file_info_for_post(self, post_id):
        return self.client.get(
            self.endpoint + '/' + post_id + '/files/info',
        )

    def get_posts_for_channel(self, channel_id, params=None):
        return self.client.get(
            Channels.endpoint + '/' + channel_id + '/posts',
            params=params
        )

    def search_for_team_posts(self, team_id, options):
        return self.client.post(
            Teams.endpoint + '/' + team_id + '/posts/search',
            options=options
        )

    def pin_post_to_channel(self, post_id):
        return self.client.post(
            self.endpoint + '/' + post_id + '/pin'
        )

    def unpin_post_to_channel(self, post_id):
        return self.client.post(
            self.endpoint + '/' + post_id + '/unpin'
        )

    def perform_post_action(self, post_id, action_id):
        return self.client.post(
            self.endpoint + '/' + post_id + '/actions/' + action_id
        )



import asyncio
import logging
import warnings

from .client import Client
from .endpoints.brand import Brand
from .endpoints.channels import Channels
from .endpoints.cluster import Cluster
from .endpoints.commands import Commands
from .endpoints.compliance import Compliance
from .endpoints.data_retention import DataRetention
from .endpoints.elasticsearch import Elasticsearch
from .endpoints.emoji import Emoji
from .endpoints.files import Files
from .endpoints.ldap import LDAP
from .endpoints.oauth import OAuth
from .endpoints.posts import Posts
from .endpoints.preferences import Preferences
from .endpoints.saml import SAML
from .endpoints.system import System
from .endpoints.teams import Teams
from .endpoints.users import Users
from .endpoints.webhooks import Webhooks
from .websocket import Websocket

log = logging.getLogger('mattermostdriver.api')
log.setLevel(logging.INFO)


class Driver:
    """
    Contains the client, api and provides you with functions for
    login, logout and initializing a websocket connection.
    """

    default_options = {
        'scheme': 'https',
        'url': 'localhost',
        'port': 8065,
        'basepath': '/api/v4',
        'verify': True,
        'timeout': 30,
        'login_id': None,
        'password': None,
        'token': None,
        'mfa_token': None
    }
    """
    Required:
        Either
        - login_id
        - password
        Or
        - token (https://docs.mattermost.com/developer/personal-access-tokens.html)

    Optional:
        - scheme
        - url (though it would be a good idea to change that)
        - port
        - verify
        - timeout
        - mfa_token
    
    Should not be changed:
        - basepath - unlikeliy this would do any good
    """

    def __init__(self, options=default_options, client_cls=Client):
        """
        :param options: A dict with the values from `default_options`
        :type options: dict
        """
        if options is None:
            options = self.default_options
        self.options = self.default_options.copy()
        self.options.update(options)
        self.driver = self.options
        self.client = client_cls(self.options)
        self._api = {
            'users': Users(self.client),
            'teams': Teams(self.client),
            'channels': Channels(self.client),
            'posts': Posts(self.client),
            'files': Files(self.client),
            'preferences': Preferences(self.client),
            'emoji': Emoji(self.client),
            'system': System(self.client),
            'webhooks': Webhooks(self.client),
            'commands': Commands(self.client),
            'compliance': Compliance(self.client),
            'cluster': Cluster(self.client),
            'brand': Brand(self.client),
            'oauth': OAuth(self.client),
            'saml': SAML(self.client),
            'ldap': LDAP(self.client),
            'elasticsearch': Elasticsearch(self.client),
            'data_retention': DataRetention(self.client),
        }
        self.websocket = None

    @asyncio.coroutine
    def init_websocket(self, event_handler, websocket_cls=Websocket, args=None):
        """
        Will initialize the websocket connection to the mattermost server.

        This should be run after login(), because the websocket needs to make
        an authentification.

        See https://api.mattermost.com/v4/#tag/WebSocket for which
        websocket events mattermost sends.

        :param event_handler: The function to handle the websocket events.
        :type event_handler: Function
        :return:
        """
        self.websocket = websocket_cls(self.options, self.client.token)
        yield from (self.websocket.connect(event_handler))

    @asyncio.coroutine
    def login(self):
        """
        Logs the user in.

        The log in information is saved in the client:

        - userid
        - username
        - cookies

        :return: The raw response from the request
        """
        if self.options['token']:
            self.client.token = self.options['token']
            result = yield from self.users.get_user('me')
        else:
            result = yield from self.users.login_user({
                'login_id': self.options['login_id'],
                'password': self.options['password'],
                'token': self.options['mfa_token']
            })
            if result.status == 200:
                self.client.token = result.headers['Token']
                self.client.cookies = result.cookies

        log.debug(result)

        json = yield from result.json()

        if 'id' in json:
            self.client.userid = json['id']
        if 'username' in json:
            self.client.username = json['username']

        return result

    def logout(self):
        """
        Log the user out.

        :return: The JSON response from the server
        """
        self.client.token = ''
        self.client.userid = ''
        self.client.username = ''
        self.client.cookies = None
        return self.users.logout_user()

    @property
    def api(self):
        warnings.warn('Deprecated for 5.0.0. Use the endpoints directly instead.', DeprecationWarning)
        return self._api

    @property
    def users(self):
        return Users(self.client)

    @property
    def teams(self):
        return Teams(self.client)

    @property
    def channels(self):
        return Channels(self.client)

    @property
    def posts(self):
        return Posts(self.client)

    @property
    def files(self):
        return Files(self.client)

    @property
    def preferences(self):
        return Preferences(self.client)

    @property
    def emoji(self):
        return Emoji(self.client)

    @property
    def system(self):
        return System(self.client)

    @property
    def webhooks(self):
        return Webhooks(self.client)

    @property
    def compliance(self):
        return Compliance(self.client)

    @property
    def cluster(self):
        return Cluster(self.client)

    @property
    def brand(self):
        return Brand(self.client)

    @property
    def oauth(self):
        return OAuth(self.client)

    @property
    def saml(self):
        return SAML(self.client)

    @property
    def ldap(self):
        return LDAP(self.client)

    @property
    def elasticsearch(self):
        return Elasticsearch(self.client)

    @property
    def data_retention(self):
        return DataRetention(self.client)



from .base import Base


class LDAP(Base):
    endpoint = '/ldap'

    def sync_ldap(self):
        return self.client.post(
            self.endpoint + '/sync'
        )

    def test_ldap_config(self):
        return self.client.post(
            self.endpoint + '/test'
        )



import logging

from .base import Base
from .teams import Teams
from .users import Users

log = logging.getLogger('mattermostdriver.api.channels')
log.setLevel(logging.INFO)


class Channels(Base):
    endpoint = '/channels'

    def create_channel(self, options):
        return self.client.post(
            self.endpoint,
            options=options
        )

    def create_direct_message_channel(self, options):
        return self.client.post(
            self.endpoint + '/direct',
            options=options
        )

    def create_group_message_channel(self, options):
        return self.client.post(
            self.endpoint + '/group',
            options=options
        )

    def get_list_of_channels_by_ids(self, team_id, options=None):
        return self.client.post(
            Teams.endpoint + '/' + team_id + '/channels/ids',
            options=options
        )

    def get_channel(self, channel_id):
        return self.client.get(
            self.endpoint + '/' + channel_id
        )

    def update_channel(self, channel_id, options):
        return self.client.put(
            self.endpoint + '/' + channel_id,
            options=options
        )

    def delete_channel(self, channel_id):
        return self.client.delete(
            self.endpoint + '/' + channel_id
        )

    def patch_channel(self, channel_id, options):
        return self.client.put(
            self.endpoint + '/' + channel_id + '/patch',
            options=options
        )

    def restore_channel(self, channel_id):
        return self.client.post(
            self.endpoint + '/' + channel_id + '/restore',
        )

    def get_channel_statistics(self, channel_id):
        return self.client.get(
            self.endpoint + '/' + channel_id + '/stats',
        )

    def get_channel_pinned_posts(self, channel_id):
        return self.client.get(
            self.endpoint + '/' + channel_id + '/pinned'
        )

    def get_channel_by_name(self, team_id, channel_name):
        return self.client.get(
            Teams.endpoint + '/' + team_id + '/channels/name/' + channel_name
        )

    def get_channel_by_name_and_team_name(self, team_name, channel_name):
        return self.client.get(
            Teams.endpoint + '/name/' + team_name + '/channels/name/' + channel_name
        )

    def get_channel_members(self, channel_id, params=None):
        return self.client.get(
            self.endpoint + '/' + channel_id + '/members',
            params=params
        )

    def add_user(self, channel_id, options=None):
        return self.client.post(
            self.endpoint + '/' + channel_id + '/members',
            options=options
        )

    def get_channel_members_by_ids(self, channel_id, options=None):
        return self.client.post(
            self.endpoint + '/' + channel_id + '/members/ids',
            options=options
        )

    def get_channel_member(self, channel_id, user_id):
        return self.client.get(
            self.endpoint + '/' + channel_id + '/members/' + user_id
        )

    def remove_channel_member(self, channel_id, user_id):
        return self.client.delete(
            self.endpoint + '/' + channel_id + '/members/' + user_id
        )

    def update_channel_roles(self, channel_id, user_id, options):
        return self.client.put(
            self.endpoint + '/' + channel_id + '/members/' + user_id + '/roles',
            options=options
        )

    def update_channel_notifications(self, channel_id, user_id, options=None):
        return self.client.put(
            self.endpoint + '/' + channel_id + '/members/' + user_id + '/notify_props',
            options=options
        )

    def view_channel(self, user_id, options):
        return self.client.post(
            self.endpoint + '/members/' + user_id + '/view',
            options=options
        )

    def get_channel_members_for_user(self, user_id, team_id):
        return self.client.get(
            Users.endpoint + '/' + user_id + '/teams/' + team_id + '/channels/members'
        )

    def get_channels_for_user(self, user_id, team_id):
        return self.client.get(
            Users.endpoint + '/' + user_id + '/teams/' + team_id + '/channels'
        )

    def get_channel_for_user(self, user_id, team_id):
        log.warning(
            'Call to deprecated function get_channel_for_user, '
            'which will be removed in the next major version.'
            'Use get_channels_for_user instead.'
        )
        return self.get_channels_for_user(user_id, team_id)

    def get_unread_messages(self, user_id, channel_id):
        return self.client.get(
            Users.endpoint + '/' + user_id + '/channels/' + channel_id + '/unread'
        )

    def get_public_channels(self, team_id, params=None):
        return self.client.get(
            '/teams/' + team_id + '/channels',
            params=params
        )

    def get_deleted_channels(self, team_id, params=None):
        return self.client.get(
            '/teams/' + team_id + '/channels/deleted',
            params=params
        )

    def search_channels(self, team_id, options=None):
        return self.client.post(
            '/teams/' + team_id + '/channels/search',
            options=options
        )



from .base import Base


class Compliance(Base):
    endpoint = '/compliance'

    def create_report(self):
        return self.client.post(
            self.endpoint + '/reports'
        )

    def get_reports(self, params=None):
        return self.client.get(
            self.endpoint + '/reports',
            params=params
        )

    def get_report(self, report_id):
        return self.client.get(
            self.endpoint + '/reports/' + report_id
        )

    def download_report(self, report_id):
        return self.client.get(
            self.endpoint + '/reports/' + report_id + '/download'
        )



from .base import Base


class Users(Base):
    endpoint = '/users'

    def login_user(self, options):
        return self.client.make_request('post', self.endpoint + '/login', options)

    def logout_user(self):
        return self.client.post(self.endpoint + '/logout')

    def create_user(self, options=None):
        return self.client.post(self.endpoint, options)

    def get_users(self, params=None):
        return self.client.get(self.endpoint, params=params)

    def get_users_by_ids(self, options=None):
        return self.client.post(
            self.endpoint + '/ids',
            options
        )

    def get_users_by_usernames(self, options=None):
        return self.client.post(
            self.endpoint + '/usernames',
            options
        )

    def search_users(self, options=None):
        return self.client.post(
            self.endpoint + '/search',
            options
        )

    def autocomplete_users(self, params=None):
        return self.client.get(
            self.endpoint + '/autocomplete',
            params=params
        )

    def get_user(self, user_id):
        return self.client.get(
            self.endpoint + '/' + user_id
        )

    def update_user(self, user_id, options=None):
        return self.client.put(
            self.endpoint + '/' + user_id,
            options
        )

    def deactivate_user(self, user_id):
        return self.client.delete(
            self.endpoint + '/' + user_id
        )

    def patch_user(self, user_id, options=None):
        return self.client.put(
            self.endpoint + '/' + user_id + '/patch',
            options
        )

    def update_user_role(self, user_id, options=None):
        return self.client.put(
            self.endpoint + '/' + user_id + '/roles',
            options
        )

    def update_user_active_status(self, user_id, options=None):
        return self.client.put(
            self.endpoint + '/' + user_id + '/active',
            options
        )

    def get_user_profile_image(self, user_id):
        return self.client.get(
            self.endpoint + '/' + user_id + '/image'
        )

    def set_user_profile_image(self, user_id, files):
        return self.client.post(
            self.endpoint + '/' + user_id + '/image',
            files=files
        )

    def get_user_by_username(self, username):
        return self.client.get(
            self.endpoint + '/username/' + username
        )

    def reset_password(self, options=None):
        return self.client.post(
            self.endpoint + '/password/reset',
            options
        )

    def update_user_mfa(self, user_id, options=None):
        return self.client.put(
            self.endpoint + '/' + user_id + '/mfa',
            options
        )

    def generate_mfa(self, user_id):
        return self.client.post(
            self.endpoint + '/' + user_id + '/mfa/generate'
        )

    def check_mfa(self, options=None):
        return self.client.post(
            self.endpoint + '/mfa',
            options
        )

    def update_user_password(self, user_id, options=None):
        return self.client.put(
            self.endpoint + '/' + user_id + '/password',
            options
        )

    def send_password_reset_mail(self, options=None):
        return self.client.post(
            self.endpoint + '/password/reset/send',
            options
        )

    def get_user_by_email(self, email):
        return self.client.get(
            self.endpoint + '/email/' + email
        )

    def get_user_sessions(self, user_id):
        return self.client.get(
            self.endpoint + '/' + user_id + '/sessions'
        )

    def revoke_user_session(self, user_id, options=None):
        return self.client.post(
            self.endpoint + '/' + user_id + '/sessions/revoke',
            options
        )

    def revoke_all_user_sessions(self, user_id):
        return self.client.post(
            self.endpoint + '/' + user_id + '/sessions/revoke/all',
        )

    def attach_mobile_device(self, options=None):
        return self.client.put(
            self.endpoint + '/sessions/device',
            options
        )

    def get_user_audits(self, user_id):
        return self.client.get(
            self.endpoint + '/' + user_id + '/audits'
        )

    def verify_user_email(self, options=None):
        return self.client.post(
            self.endpoint + '/email/verify',
            options
        )

    def send_verification_mail(self, options=None):
        return self.client.post(
            self.endpoint + '/email/verify/send',
            options
        )

    def switch_login_method(self, options=None):
        return self.client.post(
            self.endpoint + '/login/switch',
            options
        )

    def disable_personal_access_token(self, options=None):
        return self.client.post(
            self.endpoint + '/tokens/disable',
            options
        )

    def enable_personal_access_token(self, options=None):
        return self.client.post(
            self.endpoint + '/tokens/enable',
            options
        )



import os.path
import sys

from setuptools import setup, find_packages

full_version = ''

root_dir = os.path.abspath(os.path.dirname(__file__))

readme_file = os.path.join(root_dir, 'README.rst')
with open(readme_file, encoding='utf-8') as f:
    long_description = f.read()

version_module = os.path.join(root_dir, 'src', 'mattermostdriver', 'version.py')
with open(version_module, encoding='utf-8') as f:
    exec(f.read())

py_version = sys.version_info[:2]

if py_version < (3, 4):
    raise Exception("python-mattermost-driver requires Python >= 3.4.")

setup(
    name='mattermostdriver-asyncio',
    version=full_version,
    description='An asyncio Python Mattermost Driver',
    long_description=long_description,
    author='Nate Mara',
    author_email='nate.mara@kroger.com',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    package_dir={'': 'src'},
    packages=find_packages('src'),
    install_requires=[
        'aiohttp>=2.3.4',
        'async-timeout>=1.4.0',
        'aiodns>=1.1.1',
        'cchardet>=2.1.1',
        'websockets>=3.2',
    ],
)



from .base import Base


class DataRetention(Base):
    endpoint = '/data_retention'

    def get_data_retention_policy(self):
        return self.client.get(
            self.endpoint + '/policy'
        )



from .base import Base


class SAML(Base):
    endpoint = '/saml'

    def get_metadata(self):
        return self.client.get(
            self.endpoint + '/metadata'
        )

    def upload_idp_certificate(self, files):
        return self.client.post(
            self.endpoint + '/certificate/idp',
            files=files
        )

    def remove_idp_certificate(self):
        return self.client.delete(
            self.endpoint + '/certificate/idp'
        )

    def upload_public_certificate(self, files):
        return self.client.post(
            self.endpoint + '/certificate/public',
            files=files
        )

    def remove_public_certificate(self):
        return self.client.delete(
            self.endpoint + '/certificate/public'
        )

    def upload_private_key(self, files):
        return self.client.post(
            self.endpoint + '/certificate/private',
            files=files
        )

    def remove_private_key(self):
        return self.client.delete(
            self.endpoint + '/certificate/private'
        )

    def get_certificate_status(self):
        return self.client.get(
            self.endpoint + '/certificate/status'
        )
